package com.tar2ga.ibox_plugin;

import android.content.res.Resources;

import java.util.Date;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import ibox.pro.sdk.external.PaymentController;
import ibox.pro.sdk.external.PaymentControllerListener;
import ibox.pro.sdk.external.PaymentException;
import ibox.pro.sdk.external.PaymentResultContext;
import io.flutter.Log;
import io.flutter.plugin.common.EventChannel;

public class IboxPaymentControllerListener implements PaymentControllerListener, EventChannel.StreamHandler {
    private Resources resources;
    private Map<String, EventChannel.EventSink> eventSinkMap = new HashMap<>();

    private static final String readerEventStr = "readerEvent";
    private static final String onBatteryStateEventStr = "onBatteryStateEvent";
    private static final String onErrorEventStr = "onErrorEvent";

    public static double twoCanChargePercent = -1d;


    public void setResources(Resources resources){
        this.resources = resources;
    }

    protected void action() throws PaymentException {

    }

    protected boolean usesReader() {
        return true;
    }

    public static void stopPayment(){
        PaymentController.getInstance().disable();
    }

    //region PaymentControllerListener
    @Override
    public void onTransactionStarted(String s) {

    }

    @Override
    public void onFinished(PaymentResultContext paymentResultContext) {

    }

    @Override
    public void onError(PaymentController.PaymentError paymentError, String errorMessage) {
        String text = "";
        if (paymentError == null){
            if (errorMessage.equals("invalid input data"))
                text = resources.getString(R.string.error_invalid_input_data);
            else
                text = errorMessage;

            setEventSink(text, onErrorEventStr);
        }
        else {
            switch (paymentError) {
                case SERVER_ERROR:
                    text = errorMessage;
                    break;
                case CONNECTION_ERROR:
                    text = resources.getString(R.string.error_no_response);
                    break;
                case EMV_NOT_ALLOWED:
                    text = resources.getString(R.string.EMV_NOT_ALLOWED);
                    break;
                case NFC_NOT_ALLOWED:
                    text = resources.getString(R.string.NFC_NOT_ALLOWED);
                    break;
                case EMV_CANCEL:
                    text = resources.getString(R.string.EMV_TRANSACTION_CANCELED);
                    break;
                case EMV_DECLINED:
                    text = resources.getString(R.string.EMV_TRANSACTION_DECLINED);
                    break;
                case EMV_TERMINATED:
                    text = resources.getString(R.string.EMV_TRANSACTION_TERMINATED);
                    break;
                case EMV_CARD_ERROR:
                    text = resources.getString(R.string.EMV_CARD_ERROR);
                    break;
                case EMV_DEVICE_ERROR:
                    text = resources.getString(R.string.EMV_READER_ERROR);
                    break;
                case EMV_CARD_BLOCKED:
                    text = resources.getString(R.string.EMV_CARD_BLOCKED);
                    break;
                case EMV_CARD_NOT_SUPPORTED:
                    text = resources.getString(R.string.EMV_CARD_NOT_SUPPORTED);
                    break;
                case NO_SUCH_TRANSACTION:
                    text = resources.getString(R.string.error_no_such_transaction);
                    break;
                case TRANSACTION_NULL_OR_EMPTY:
                    text = resources.getString(R.string.error_tr_null_or_empty);
                    break;
                case INVALID_INPUT_TYPE:
                    text = resources.getString(R.string.error_invalid_input_type);
                    break;
                case INVALID_AMOUNT:
                    text = resources.getString(R.string.error_invalid_amount);
                    break;
                case TTK_FAILED:
                    text = String.format(resources.getString(R.string.error_standalone_format), errorMessage);
                    break;
                default:
                    text = resources.getString(R.string.EMV_ERROR);
                    break;
            }

            setEventSink(text, onErrorEventStr);
        }
    }

    @Override
    public void onReaderEvent(PaymentController.ReaderEvent readerEvent) {
        switch (readerEvent) {
            case CONNECTED:
                setEventSink(R.string.connected, readerEventStr);
                break;
            case START_INIT:
                setEventSink(R.string.startInit, readerEventStr);
                break;
            case INIT_SUCCESSFULLY:
                setEventSink(R.string.initSuccess, readerEventStr);
                //PaymentController.getInstance().readerBeep(2);
                break;
            case DISCONNECTED:
                setEventSink(R.string.disconnected, readerEventStr);
                break;
            case SWIPE_CARD:
            case EMV_TRANSACTION_STARTED:
            case NFC_TRANSACTION_STARTED:
                break;
            case WAITING_FOR_CARD:
                setEventSink(R.string.reader_state_ready_multiinput, readerEventStr);
                break;
            case PAYMENT_CANCELED:
                setEventSink(R.string.payment_dlg_canceled, readerEventStr);
                stopPayment();
                break;
            case INIT_FAILED:
                setEventSink(R.string.reader_state_init_error, readerEventStr);
                break;
            case EJECT_CARD:
                //перекрывает сообщение об ошибке
                break;
            case BAD_SWIPE:
                setEventSink(R.string.reader_bad_swipe, readerEventStr);
                break;
            case LOW_BATTERY:
                setEventSink(R.string.reader_low_battery, readerEventStr);
                break;
            case CARD_TIMEOUT:
                setEventSink(R.string.reader_card_timeout, readerEventStr);
                stopPayment();
                break;
            case PIN_TIMEOUT:
                setEventSink(R.string.reader_pin_timeout, readerEventStr);
                stopPayment();
                break;
            default:
                break;
        }
    }

    @Override
    public int onSelectApplication(List<String> list) {
        return 0;
    }

    @Override
    public boolean onConfirmSchedule(List<Map.Entry<Date, Double>> list, double v) {
        return false;
    }

    @Override
    public boolean onScheduleCreationFailed(PaymentController.PaymentError paymentError, String s) {
        return false;
    }

    @Override
    public boolean onCancellationTimeout() {
        return false;
    }

    @Override
    public void onPinRequest() {

    }

    @Override
    public void onPinEntered() {

    }

    @Override
    public void onAutoConfigUpdate(double v) {

    }

    @Override
    public void onAutoConfigFinished(boolean b, String s, boolean b1) {

    }

    @Override
    public void onBatteryState(double v) {
        twoCanChargePercent = v;
        setEventSink(R.string.batteryСharge, onBatteryStateEventStr);
    }

    @Override
    public PaymentController.PaymentInputType onSelectInputType(List<PaymentController.PaymentInputType> list) {
        return null;
    }

    @Override
    public void onSwitchedToCNP() {

    }

    @Override
    public void onSearchMifareCard(Hashtable<String, String> hashtable) {

    }

    @Override
    public void onVerifyMifareCard(boolean b) {

    }

    @Override
    public void onWriteMifareCard(boolean b) {

    }

    @Override
    public void onReadMifareCard(Hashtable<String, String> hashtable) {

    }

    @Override
    public void onOperateMifareCard(Hashtable<String, String> hashtable) {

    }

    @Override
    public void onTransferMifareData(String s) {

    }

    @Override
    public void onFinishMifareCard(boolean b) {

    }

    @Override
    public void onReturnPowerOnNFCResult(boolean b) {

    }

    @Override
    public void onReturnNFCApduResult(boolean b, String s, int i) {

    }

    @Override
    public void onReturnPowerOffNFCResult(boolean b) {

    }

    //endregion

    private void setEventSink(int resId, String eventType){
        Log.d(resources.getString(R.string.twoCanEvent), resources.getString(resId));

        EventChannel.EventSink eventSinkEvent = eventSinkMap.get(eventType);
        if (eventSinkEvent != null) {
            switch (eventType) {
                case readerEventStr:
                    eventSinkEvent.success(resources.getString(resId));
                    break;
                case onBatteryStateEventStr:
                    eventSinkEvent.success(twoCanChargePercent);
                    break;
            }
        }
    }

    private void setEventSink(String message, String eventType){
        Log.d(resources.getString(R.string.twoCanEvent), message);

        EventChannel.EventSink eventSinkEvent = eventSinkMap.get(eventType);
        if (eventSinkEvent != null) {
            switch (eventType) {
                case onErrorEventStr:
                    eventSinkEvent.error("-1", message, null);
                    break;
            }
        }
    }

    //region EventChannel.StreamHandler
    @Override
    public void onListen(Object arguments, EventChannel.EventSink events) {
        switch ((String)arguments){
            case readerEventStr:
                eventSinkMap.put(readerEventStr, events);
                break;
            case onBatteryStateEventStr:
                eventSinkMap.put(onBatteryStateEventStr, events);
                break;
            case onErrorEventStr:
                eventSinkMap.put(onErrorEventStr, events);
                break;
        }
    }

    @Override
    public void onCancel(Object arguments) {
        switch ((String)arguments){
            case readerEventStr:
                EventChannel.EventSink eventSinkReaderEvent = eventSinkMap.get(readerEventStr);
                eventSinkReaderEvent = null;
                break;
        }
    }

    //endregion
}
