package com.tar2ga.ibox_plugin;

import androidx.annotation.NonNull;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/** IboxPlugin */
public class IboxPlugin extends IboxLogic implements FlutterPlugin, MethodCallHandler , ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;
  private EventChannel readerEventChanel;
  private EventChannel batteryStateEventChanel;
  private EventChannel errorEventChanel;

  private static IboxPaymentControllerListener iboxPaymentControllerListener;

  private static final String channelName = "ibox_pro_sdk_external";
  private static final String readerEventChannelName = "ibox_reader_event";
  private static final String batteryStateEventChannelName = "ibox_battery_state_event";
  private static final String errorEventChannelName = "ibox_error_event";

  // This static function is optional and equivalent to onAttachedToEngine. It supports the old
  // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
  // plugin registration via this function while apps migrate to use the new Android APIs
  // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
  //
  // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
  // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
  // depending on the user's project. onAttachedToEngine or registerWith must both be defined
  // in the same class.
  public static void registerWith(Registrar registrar) {
    IboxPlugin plugin = new IboxPlugin();

    final MethodChannel channel = new MethodChannel(registrar.messenger(), channelName);
    channel.setMethodCallHandler(plugin);

    final EventChannel readerEventChanel = createEventChannel(registrar, readerEventChannelName);
    final EventChannel batteryStateEventChannel = createEventChannel(registrar, batteryStateEventChannelName);
    final EventChannel errorStateEventChannel = createEventChannel(registrar, errorEventChannelName);
  }

  public static IboxPaymentControllerListener getListener(){
    IboxPaymentControllerListener localListener = iboxPaymentControllerListener;
    if (localListener == null){
      synchronized (IboxPaymentControllerListener.class){
        localListener = iboxPaymentControllerListener;
        if (localListener == null){
          iboxPaymentControllerListener = localListener = new IboxPaymentControllerListener();
        }
      }
    }
    return localListener;
  }

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    IboxPlugin plugin = new IboxPlugin();

    channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), channelName);
    channel.setMethodCallHandler(plugin);

    setEventChannel(flutterPluginBinding, readerEventChanel, readerEventChannelName);
    setEventChannel(flutterPluginBinding, batteryStateEventChanel, batteryStateEventChannelName);
    setEventChannel(flutterPluginBinding, errorEventChanel, errorEventChannelName);
  }

  private void setEventChannel(@NonNull FlutterPluginBinding flutterPluginBinding, EventChannel eventChannel, String channelName){
    eventChannel = new EventChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), channelName);
    eventChannel.setStreamHandler(getListener());
  }

  private static EventChannel createEventChannel(Registrar registrar, String channelName){
    final EventChannel eventChannel = new EventChannel(registrar.messenger(), channelName);
    eventChannel.setStreamHandler(getListener());
    return eventChannel;
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
    readerEventChanel.setStreamHandler(null);
    batteryStateEventChanel.setStreamHandler(null);
    errorEventChanel.setStreamHandler(null);
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    switch (call.method){
      case "getPlatformVersion":
        result.success("Android " + android.os.Build.VERSION.RELEASE);
        break;
      case "enable2can":
        enable2can(call, result);
        break;
      case "attachCard":
        attachCard(call, result);
        break;
      case "auth":
        auth(call, result);
        break;
      default:
        result.notImplemented();
    }
  }

  @Override
  public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
    configureResources(binding);
  }

  @Override
  public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
    configureResources(binding);
  }

  private void configureResources(@NonNull ActivityPluginBinding binding){
    activity = binding.getActivity();
    getListener().setResources(activity.getResources());
  }

  @Override
  public void onDetachedFromActivity() {

  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {

  }
}
