
import 'dart:async';

import 'package:flutter/services.dart';

class IboxPlugin {
  static const MethodChannel _channel = const MethodChannel('ibox_pro_sdk_external');
  static const EventChannel _readerEventChannel = const EventChannel("ibox_reader_event");
  static const EventChannel _batteryStateEventChannel = const EventChannel("ibox_battery_state_event");
  static const EventChannel _errorEventChannel = const EventChannel("ibox_error_event");

  static final String readerEventStr = "readerEvent";
  static final String onBatteryStateEventStr = "onBatteryStateEvent";
  static final String onErrorEventStr = "onErrorEvent";

  static Future<String> get enable2can async {
    final String version = await _channel.invokeMethod('enable2can', <String, dynamic>{'ss' : 33, 'v': "test"});
    //await _channel.invokeMethod('enable2can');
    //await _channel.invokeMethod('enable2can', <String, dynamic>{'ss' : 33, 'v': "test"});
    return version;
  }

  static Future<String> auth(String login, String secretKey) async {
    final String accountStr = await _channel.invokeMethod('auth', <String, dynamic>{'login' : login, 'secretKey': secretKey});
    return accountStr;
  }

  static Future<void> get attachCard async {
    await _channel.invokeMethod('attachCard');
  }

  static Stream<String> _ReaderEventStream;
  static Stream<String> get getReaderEventStream {
    if (_ReaderEventStream == null){
      _ReaderEventStream = _readerEventChannel.receiveBroadcastStream(readerEventStr).map<String>((value) => value); //map - конвертирует значения в указанный тип "на лету"
    }
    return _ReaderEventStream;
  }

  static Stream<double> _batteryStateEventStream;
  static Stream<double> get getBatteryStateEventStream {
    if (_batteryStateEventStream == null){
      _batteryStateEventStream = _batteryStateEventChannel.receiveBroadcastStream(onBatteryStateEventStr).map<double>((value) => value);
    }
    return _batteryStateEventStream;
  }

  static Stream<String> _errorEventStream;
  static Stream<String> get getErrorEventStream {
    if (_errorEventStream == null){
      _errorEventStream = _errorEventChannel.receiveBroadcastStream(onErrorEventStr).map<String>((value) => value); //map - конвертирует значения в указанный тип "на лету"
    }
    return _errorEventStream;
  }
}
